#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
import zmq

# import interfaces
from pyams_scheduler.interfaces import IScheduler, ISchedulerHandler, SCHEDULER_HANDLER_KEY
from zope.intid.interfaces import IIntIds

# import packages
from pyams_utils.registry import query_utility
from pyramid.threadlocal import get_current_registry
from zope.container.folder import Folder
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty


@implementer(ISchedulerHandler)
class SchedulerHandler(object):
    """Scheduler handler utility

    This is just a 'marker' utility which is used to mark nodes in a cluster
    which should run the scheduler
    """


@implementer(IScheduler)
class Scheduler(Folder):
    """Scheduler utility"""

    zeo_connection = FieldProperty(IScheduler['zeo_connection'])
    report_mailer = FieldProperty(IScheduler['report_mailer'])
    report_source = FieldProperty(IScheduler['report_source'])

    @property
    def tasks(self):
        return list(self.values())

    @property
    def history(self):
        result = []
        [result.extend(task.history) for task in self.values()]
        result.sort(key=lambda x: x.date)
        return result

    @property
    def internal_id(self):
        intids = query_utility(IIntIds, context=self)
        if intids is not None:
            return intids.register(self)

    def get_task(self, task_id):
        intids = query_utility(IIntIds, context=self)
        if intids is not None:
            return intids.queryObject(task_id)

    def _get_socket(self):
        """Open ØMQ socket"""
        registry = get_current_registry()
        handler = registry.settings.get(SCHEDULER_HANDLER_KEY, False)
        if handler:
            context = zmq.Context()
            socket = context.socket(zmq.REQ)
            socket.connect('tcp://{0}'.format(handler))
            return socket

    def get_jobs(self):
        socket = self._get_socket()
        socket.send_json(['get_jobs', {}])
        return socket.recv_json()
